package edu.cmu.cs211.graph;
import java.awt.GraphicsEnvironment;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Formatter;

// Thanks to Owen Yamauchi <ody@andrew.cmu.edu> for contributing
// this code.

/**
 * Formats a graph for <q>dot</q> output.
 */
public class GraphVisualizer {
	
	public static <V, E extends Edge<V>> void outputGraph (Graph<V,E> g, String f)
	{
		try {
			outputGraph(g, new Formatter (f));
		} catch (FileNotFoundException e) {
			throw new RuntimeException (e);
		}
	}
	
	/**
	 * Generates an image of a given graph, then displays it
	 */
	public static  <V, E extends Edge<V>> void displayGraph (Graph<V,E> g)
	{
		// make sure this isn't used on FrontDesk
		if (GraphicsEnvironment.isHeadless ())
			return;

		try {
			File dot = File.createTempFile ("dotout", ".dot");

			outputGraph (g, dot.getAbsolutePath ());

			File png = File.createTempFile ("dotout", ".png");
			run ("dot", "-Tpng", "-o", png.getAbsolutePath (), dot
			        .getAbsolutePath ());

			// eog is an image viewer. It only exists in the course
			// VMWare image. If you don't have this, enter the path
			// to a program that displays images.
			run ("eog", png.getAbsolutePath ());
			dot.delete ();
			png.delete ();
		} catch (Exception ex) {
			throw new RuntimeException (ex);
		}
	}

	private static void run (String... args) throws InterruptedException,
	        IOException
	{
		new ProcessBuilder (args).start ().waitFor ();
	}
	
	public static <V,E extends Edge<V>> void outputGraph (Graph<V,E> g, Formatter f)
	{
		f.format ("digraph {\n");
		for (V vertex : g.vertices()) {
			f.format ("\tn%s [label=%s]\n", vertex , vertex);
		}
		
		for (V vertex : g.vertices())
			for (E e : g.outgoingEdges(vertex))
				f.format ("\tn%s -> n%s [label=%d,weight=%d]\n", e.src(), e.dest(),e.weight(),e.weight());
		
		f.format("}\n");
		f.close ();
	}
}
